-- Returns grand total 
SELECT SUM(TotalDue) AS GrandTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Returns grand total once for each row of input
SELECT SUM(TotalDue) OVER() AS GrandTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Returns grand total once for each row of input
SELECT SalesOrderID, 
	TotalDue,
	SUM(TotalDue) OVER() AS GrandTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Non-window way to do it
SELECT 
	SalesOrderID,
	TotalDue,
	a.GrandTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader
CROSS APPLY (SELECT SUM(TotalDue) AS GrandTotal
		FROM AdventureWorks2012.Sales.SalesOrderHeader
		) AS a

-- Now make it meaningful
SELECT SalesOrderID, 
	TotalDue,
	SUM(TotalDue) OVER() AS GrandTotal,
	(TotalDue/SUM(TotalDue) OVER())*100 AS PctOfTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Combining window AND grouped by functions in the same query
-- Now, let's try....  WHAT?????  Why?
SELECT CustomerID,
	SUM(TotalDue) AS CustomerTotal,
	SUM(TotalDue) OVER() AS GrandTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader
GROUP BY CustomerID

-- Remember the order the query is processed
SELECT CustomerID,
	SUM(TotalDue) AS CustomerTotal,
	SUM(SUM(TotalDue)) OVER() AS GrandTotal,
	(SUM(TotalDue)/SUM(SUM(TotalDue)) OVER()) AS PctOfTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader
GROUP BY CustomerID

-- Is there a better way?
-- Of course!
SELECT CustomerID,
	SUM(TotalDue) OVER(PARTITION BY CustomerID) AS CustomerTotal,
	SUM(TotalDue) OVER() AS GrandTotal,
	(SUM(TotalDue) OVER(PARTITION BY CustomerID)/SUM(TotalDue) OVER()) AS PctOfTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Now let's get creative
-- Here's what we have to work with
SELECT CustomerID,
	OrderDate,
	TotalDue
FROM AdventureWorks2012.Sales.SalesOrderHeader
ORDER BY CustomerID, OrderDate

-- This is what we can get using the grouped by function
SELECT CustomerID,
	SUM(TotalDue) CustomerRunningTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader
GROUP BY CustomerID
ORDER BY CustomerID

-- But what if that's not really what you need?
-- Let's get a running total by customer AND date, AND show the order amount for each date along with the total
SELECT CustomerID,
	OrderDate,
	TotalDue,
	SUM(TotalDue) OVER(PARTITION BY CustomerID ORDER BY OrderDate) AS CustomerRunningTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader
ORDER BY CustomerID, OrderDate

-- Now, let's add a running total by date for all customers to the mix
SELECT CustomerID,
	OrderDate,
	TotalDue,
	SUM(TotalDue) OVER(PARTITION BY CustomerID ORDER BY OrderDate) AS CustomerRunningTotal,
	SUM(TotalDue) OVER(ORDER BY OrderDate) AS AllCustomersRunningTotal
FROM AdventureWorks2012.Sales.SalesOrderHeader
ORDER BY CustomerID, OrderDate
--ORDER BY OrderDate, CustomerID

-- Back off some of the detail AND mix grouped AND window functions
SELECT CustomerID,
	SUM(TotalDue) AS CustomerTotal,				--this is the customer's individual total
	SUM(SUM(TotalDue)) OVER() AS GrandTotal,	--this is the total of all orders, note the grouped functiON embedded in the window function,
	(SUM(TotalDue)/SUM(SUM(TotalDue)) OVER())*100 AS PctOfTotal  --now divide the grouped functiON by the window(grouped) function
FROM AdventureWorks2012.Sales.SalesOrderHeader
WHERE OrderDate = '2005-07-01 00:00:00.000'  --use only one day so the numbers are manageable
GROUP BY CustomerID
ORDER BY CustomerID
--ORDER BY PctOfTotal desc  --see what I did here?  remember the order a query is processed


-- Did you realize you just got framed here???  No....



-- Basic syntax introduced in SQL Server 2005 but with limitations
SELECT 
	AccountNumber,
	OrderDate,
	ROW_NUMBER() OVER (ORDER BY OrderDate) AS RowID
FROM AdventureWorks2012.Sales.SalesOrderHeader
ORDER BY OrderDate

-- Problems in order can affect the output when indescriminate
SELECT 
	AccountNumber,
	OrderDate,
	ROW_NUMBER() OVER (ORDER BY OrderDate) AS RowID
FROM AdventureWorks2012.Sales.SalesOrderHeader
ORDER BY OrderDate, AccountNumber

-- Not quite the same thing.....
SELECT 
	AccountNumber,
	OrderDate,
	ROW_NUMBER() OVER (ORDER BY OrderDate) AS RowID
FROM AdventureWorks2012.Sales.SalesOrderHeader
ORDER BY AccountNumber;

-- Still differnet......
SELECT 
	AccountNumber,
	OrderDate,
	ROW_NUMBER() OVER (ORDER BY AccountNumber, OrderDate) AS RowID
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Partitioning by AccountNumber
SELECT 
	AccountNumber,
	OrderDate,
	ROW_NUMBER() OVER (PARTITION BY AccountNumber ORDER BY OrderDate) AS RowID
FROM AdventureWorks2012.Sales.SalesOrderHeader


-- Partitioning by OrderDate
SELECT 
	AccountNumber,
	OrderDate,
	ROW_NUMBER() OVER (PARTITION BY OrderDate ORDER BY SalesOrderID) AS RowID
FROM AdventureWorks2012.Sales.SalesOrderHeader




SELECT 
	AccountNumber,
	OrderDate, 
	FIRST_VALUE(OrderDate) OVER (PARTITION BY AccountNumber ORDER BY OrderDate) AS FirstOrderDate,
	LAST_VALUE(OrderDate) OVER (PARTITION BY AccountNumber ORDER BY OrderDate) AS LastOrderDate
FROM AdventureWorks2012.Sales.SalesOrderHeader


-- Let's see how Framing helps here
SELECT 
	AccountNumber,
	OrderDate, 
	First_VALUE(OrderDate) 
		OVER (	PARTITION BY AccountNumber 
				ORDER BY OrderDate 
				ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS FirstOrderDate,
	LAST_VALUE(OrderDate) 
		OVER (	PARTITION BY AccountNumber 
				ORDER BY OrderDate 
				ROWS BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING) AS LastOrderDate
FROM AdventureWorks2012.Sales.SalesOrderHeader

SET STATISTICS TIME ON
SET STATISTICS IO ON

-- Let's see how Framing helps here
SELECT 
	AccountNumber,
	OrderDate, 
	First_VALUE(OrderDate) 
		OVER (	PARTITION BY AccountNumber 
				ORDER BY OrderDate 
				ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS FirstOrderDate,
	LAST_VALUE(OrderDate) 
		OVER (	PARTITION BY AccountNumber 
				ORDER BY OrderDate 
				ROWS BETWEEN CURRENT ROW AND UNBOUNDED FOLLOWING) AS LastOrderDate
FROM AdventureWorks2012.Sales.SalesOrderHeader


-- Let's see how Framing helps here
SELECT 
	AccountNumber,
	OrderDate, 
	First_VALUE(OrderDate) 
		OVER (	PARTITION BY AccountNumber 
				ORDER BY OrderDate 
				ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS FirstOrderDate,
	LAST_VALUE(OrderDate) 
		OVER (	PARTITION BY AccountNumber 
				ORDER BY OrderDate 
				ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS LastOrderDate
FROM AdventureWorks2012.Sales.SalesOrderHeader

-- Let's see how we would have gotten here previously?
SELECT soh.AccountNumber, OrderDate, FirstOrderDate, LastOrderDate
FROM Sales.SalesOrderHeader AS soh
JOIN (
SELECT 
	AccountNumber,
	MIN(OrderDate) AS FirstOrderDate, 
	MAX(OrderDate) AS LastOrderDate
FROM AdventureWorks2012.Sales.SalesOrderHeader
GROUP BY AccountNumber) AS tab ON tab.AccountNumber = soh.AccountNumber


SELECT 
	AccountNumber,
	OrderDate,
	(SELECT MIN(OrderDate) FROM (SELECT 1 AS tabval) AS t GROUP BY AccountNumber, tabval) AS FirstOrderDate, 
	(SELECT MAX(OrderDate) FROM (SELECT 1 AS tabval) AS t GROUP BY AccountNumber, tabval) AS LastOrderDate
FROM AdventureWorks2012.Sales.SalesOrderHeader
GROUP BY AccountNumber


-- PERCENT_RANK
USE AdventureWorks2012;
GO
SELECT Department, LastName, Rate, 
       CUME_DIST () OVER (PARTITION BY Department ORDER BY Rate) AS CumeDist, 
       PERCENT_RANK() OVER (PARTITION BY Department ORDER BY Rate ) AS PctRank,
	   RANK() OVER (Partition BY Department ORDER BY Rate) AS [Rank],
	   COUNT(*) OVER (PARTITION BY Department) AS [Count],
	   (RANK() OVER (PARTITION BY Department ORDER BY Rate) - 1. ) / (COUNT(*) OVER (PARTITION BY Department) - 1.) AS pctRank_calc
FROM HumanResources.vEmployeeDepartmentHistory AS edh
    INNER JOIN HumanResources.EmployeePayHistory AS e  
    ON e.BusinessEntityID = edh.BusinessEntityID
WHERE Department IN (N'Shipping and Receiving') 
ORDER BY Department, Rate DESC;

USE AdventureWorks2012;
GO
SELECT Department, LastName, Rate, 
       PERCENT_RANK() OVER (PARTITION BY Department ORDER BY Rate ) AS PctRank
FROM HumanResources.vEmployeeDepartmentHistory AS edh
    INNER JOIN HumanResources.EmployeePayHistory AS e  
    ON e.BusinessEntityID = edh.BusinessEntityID
WHERE Department IN (N'Information Services',N'Document Control') 
ORDER BY Department, Rate DESC;


USE AdventureWorks2012;
GO
SELECT Department, LastName, Rate, 
	   (RANK() OVER (PARTITION BY Department ORDER BY Rate) - 1. ) / (COUNT(*) OVER (PARTITION BY Department) - 1.) AS pctRank_calc
FROM HumanResources.vEmployeeDepartmentHistory AS edh
    INNER JOIN HumanResources.EmployeePayHistory AS e  
    ON e.BusinessEntityID = edh.BusinessEntityID
WHERE Department IN (N'Information Services',N'Document Control') 
ORDER BY Department, Rate DESC;